%//==============================================================================
%//
%//  Copyright (C) 2000  Vojko Valencic <Vojko.Valencic@fe.uni-lj.si>
%//                      Savin Zlobec <savin@torina.fe.uni-lj.si>
%//
%//  This program is free software; you can redistribute it and/or
%//  modify it under the terms of the GNU General Public License
%//  as published by the Free Software Foundation; either version 2
%//  of the License, or (at your option) any later version.
%//
%//  This program is distributed in the hope that it will be useful,
%//  but WITHOUT ANY WARRANTY; without even the implied warranty of
%//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%//  GNU General Public License for more details.
%//
%//  You should have received a copy of the GNU General Public License
%//  along with this program; if not, write to the Free Software
%//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
%//
%//==============================================================================
%potential(runstr, N, filename); 
%  input: 
%		 .txt filename from JaCoB delimited by space or
%      .jco file XML like experiment script
%  			
%      N - Number of mesh divisions
%
%      runstr  (plot styles)
%			m - mesh 
% 			s - surface 
%        l - surfl (light)
%			z - meshz
%			mc - meshc
%			c  - contour
%			q  - quiver
%        p  - particles add to plot
%
%
%=================================================================================
function [V,X,Y,dx,dy,x,y] = potential(runstr, N, filename);



if nargin < 1, runstr = 'mc'; N = 40; end;

if nargin < 2,  N = 50; end;

if nargin < 3, disp('No data matrix present. Select the file');
   % help potential,
   [filename, d, s]= filenameDlgJacobHere;
   messagedisply = strcat(filename.name,' is selected');
   disp(messagedisply);
else XY = dlmread(filename,' ');
   filename.name = filename;
end;

% Read data the JaCoB generated file

if findstr(filename.name,'.txt'), 
   XY = dlmread(filename.name,' ');
elseif findstr(filename.name,'.jco'),
   XY = jcoparticleread(filename.name);
else
   disp('No correct file name is selected. Close *.txt or *.jco file');
   return
end

   xp  = XY(:,1);
   yp  = XY(:,2);
   yp = -yp;
  	qp  = XY(:,3);
	xmin = min(xp);
	xmax = max(xp);
	ymin = min(yp);
	ymax = max(yp);
	xa = (xmin + xmax)./2;
	ya = (ymin + ymax)./2;

enlargement  = 2;
	x1 = xa - (xa - xmin) .* enlargement;
	x2 = xa + (xa - xmin) .* enlargement;
 
	y1 = ya - (ya - ymin) .* enlargement;
	y2 = ya + (ya - ymin) .* enlargement;

gridnum  = N - 1;
	dx =  abs((x2 - x1)) ./ gridnum;
   dy =  abs((y2 - y1)) ./ gridnum;
   
 	eps = dx/2; % za popravek blinjih potencialov
	dy = dx;
	x  = x1:dx:x2;
	y  = y1:dy:y2;
	To = [-10*xa,ya];

   [X,Y] = meshgrid(x,y);

        rToxy = sqrt((X - To(1)) .* (X - To(1)) +...
                     (Y - To(2)) .* (Y - To(2)));
	        [rp cp] = size(XY);
           V = 0.0;
           
           I = [];
           for p = 1:rp,
           rTqxy = sqrt((X - xp(p)) .* (X - xp(p)) +...
                        (Y - yp(p)) .* (Y - yp(p)));  
           V = V - qp(p) .* log(rTqxy ./ rToxy);
         	% popravek blinjih potencialov
            oddaljenosti = rTqxy - eps;
            oddaljenosti = find(oddaljenosti < 0.0);% ideks tocke mreze, za katero je p preblizu
            if isempty(oddaljenosti) == 0
               I = [I;oddaljenosti,p,rTqxy(oddaljenosti)];% idx tocke, idx elektrine, razdalja
            end
         end
         [Ix,i] = size(I);
         
         [Xrows,Xcols] = size(X);
         Vsosed = 0;
         VsL = 0;
         for i = 1:Ix
            	qpv = qp(I(i,2)) * log( I(i,3) / rToxy(I(i,1)) );
               V(I(i,1)) = V(I(i,1)) + qpv;
            	qpv = qp(I(i,2)) * log( eps / rToxy(I(i,1)) );
               V(I(i,1)) = V(I(i,1)) - qpv;
 %                 Vsosed = Vsosed + I(i,1)-1;
 %                 VsL = VsL + 1;
 %              end
 %              if I(i,1)+1 <= Xrows
 %                 Vsosed = Vsosed + I(i,1)+1;
 %                 VsL = VsL + 1;
 %              end
 %              if I(i,1)-Xrows >= 0
 %                 Vsosed = Vsosed + I(i,1)-Xrows;
 %                 VsL = VsL + 1;
 %              end
 %              if I(i,1)+Xrows <= Xcols
 %                 Vsosed = Vsosed + I(i,1)+Xrows;
 %                 VsL = VsL + 1;
 %              end
 %              V(I(i,1)) = Vsosed / VsL;
            end
         
    clf; 
        axis('equal');
    fign = 1;

 if findstr(runstr,'c'),
    figure(fign); fign = fign + 1;
    hold on;
    contourf(x,y,V,20);
    if findstr(runstr,'p'),        
		 plotCharges(qp,xp,yp);
    end; 
    plotLabelV(filename);
 end;
  hold on;
 if findstr(runstr,'s'),
    figure(fign); fign = fign + 1;
    surf(x,y,V);
    shading interp;
    plotLabelV(filename);
 end;
 
 if findstr(runstr,'m'), 
     figure(fign); fign = fign + 1;
    mesh(x,y,V);
    plotLabelV(filename);
 end;
 
 if findstr(runstr,'l'),
     figure(fign); fign = fign + 1;
    surfl(x,y,V);
    plotLabelV(filename);
 end;
 
 if findstr(runstr,'z'),
    figure(fign); fign = fign + 1;
    meshz(x,y,V);
    plotLabelV(filename);
 end;
 
 if findstr(runstr,'i'),
    figure(fign); fign = fign + 1;
    pcolor(V), shading interp;
    plotLabelV(filename);
 end;

 
  if findstr(runstr,'mc'),        
     figure(fign); fign = fign + 1;
     meshc(x,y,V);
    plotLabelV(filename);
 end;
 
 if findstr(runstr,'q'), 
    figure(fign); fign = fign + 1;
    [dxg,dyg] = gradient(V,dx,dy);
    contour(X,Y,V,21);
    hold on;
    quiver(X,Y,-dxg,-dyg);
     if findstr(runstr,'p'),        
			plotCharges(qp,xp,yp);
     end;      
       plotLabelV(filename);
  end;
    title(strcat('JaCoB model file: ','../',filename.name));    
    	%rotate3d;
      hold off;
       
function plotCharges(qp,xp,yp);
 for p = 1:length(xp),
       if qp(p) > 0,
          plot(xp(p),yp(p),'ro');
          plot(xp(p),yp(p),'r+');
       else
          plot(xp(p),yp(p),'bo');
          plot(xp(p),yp(p),'b');
		 end;
    end;
    
function plotLabelV(filename);
	xlabel(' x /cm');
	ylabel(' y /cm');
   zlabel(' V /V');
   title(strcat('JaCoB model file:..',filename.name));
   
function [filename, d, s]= filenameDlgJacobHere;
%List selection dialog box.
%	[filenames, d, s]= filenameDlgJacob;
%	filename = filenameDlg;
fileExtStr = '*.*';
d = dir(fileExtStr);
str = {d.name};
[s,v] = listdlg('PromptString','Select a txt file name:',...
   'SlectionMode','single',...
   'ListSize',[180,160],...
   'ListString',str);
filename = d(s);


function XY = jcoparticleread(filename);
	vhod = fopen(filename);
   x = [];
	y = [];
	q = [];
  % branje
  while 1
	  while (~strncmp('<particle', fscanf(vhod,'%s',1), 4))
		end	
		fseek(vhod,-9,0);
		while  (~strncmp('</components', fscanf(vhod,'%s',1), 2))
	    fseek(vhod,4,0);
	    x = [x;fscanf(vhod,'%f',1)];
	    fseek(vhod,5,0);
			y = [y;fscanf(vhod,'%f',1)];
	    fseek(vhod,10,0);
			q = [q;fscanf(vhod,'%f',1)];
	    fseek(vhod,3,0);
    end
  	if (strncmp('</element', fscanf(vhod,'%s',1), 4))								% ko se pojavi ta
      if (strncmp('</components', fscanf(vhod,'%s',1), 4)), break, end		% kombinacija se vnos konca
		end
	end
   fclose(vhod);
   XY = [];
   XY = [x y q];
  % END branje

